#pragma once

#include "const.h"
#include <string>
#include <tchar.h>

#include "IInstallController.h"
ERRORINFO *MakeErrorInfo( int code, const TCHAR *msg );

typedef std::basic_string<TCHAR>	tstring;

enum ModuleStatus
{
	ModuleStatusINIT = 0,
	ModuleStatusPROCESS,
	ModuleStatusFINISHED,
};

void SetLauncherName( const TCHAR *name );
const TCHAR *GetLauncherName();

class IModule;
class ICallback
{
public:
	ICallback() : m_module(NULL)
	{

	}

	virtual ~ICallback()
	{
		
	}

	void SetModule( IModule *module )
	{
		m_module = module;
	}

	IModule *GetModule()
	{
		return m_module;
	}
private:
	IModule *m_module;
};
/**
* @class IModule
* PrepareModule  InstallModule  Parent Ŭ, ModuleStatus  ִ.
* ϴ ProgramInformation Ŭ.
*
* @author : DaeMyung.Kang
*/
class IModule
{
public:
	IModule(const TCHAR *guid) : m_guid(guid), m_status(ModuleStatusINIT), m_pending(FALSE), m_cancel(FALSE), m_callback(NULL), m_error(FALSE)
	{

	}

	virtual ~IModule()
	{
		delete m_callback;
		m_callback = NULL;
	}

	void SetModuleStatus( ModuleStatus status )
	{
		m_status = status;
	}

	ModuleStatus GetModuleStatus()
	{
		return m_status;
	}

	BOOL IsCancel()
	{
		return m_cancel;
	}

	void SetError()
	{
		m_error = TRUE;
	}

	BOOL IsError()
	{
		return m_error;
	}

	void Cancel()
	{
		m_pending = FALSE;
		m_cancel = TRUE;
	}

	void SetPending( BOOL pending )
	{
		m_pending = pending;
	}

	BOOL IsPending()
	{
		return m_pending;
	}

	const TCHAR *GetGuid()
	{
		return m_guid.c_str();
	}

	void SetCallback(ICallback *callback)
	{
		m_callback = callback;
		m_callback->SetModule(this);
	}
protected:
	ICallback*		m_callback;
private:
	const tstring	m_guid;
	ModuleStatus	m_status;
	BOOL			m_pending;
	BOOL			m_cancel;
	BOOL			m_error;
};